@@ -6,20 +6,27 @@ import mock
 
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
-from django.utils import unittest
+from django import test
 
 from request_signer import constants
-from request_signer.client.generic import Request, Response, Client
+from request_signer import models
+from request_signer.client.generic import Client, Response, Request, django_backend
 
-__all__ = ('ClientTests', )
 
-class ClientTests(unittest.TestCase):
+class TestableClient(Client):
+
+    def __init__(self):
+        api_credentials = django_backend.DjangoSettingsApiCredentialsBackend(self)
+        super(TestableClient, self).__init__(api_credentials)
+
+
+class ClientTests(test.TestCase):
 
     def setUp(self):
         self.settings_name = 'SAMPLE_AUTH_DOMAIN'
         self.client_id_settings_name = 'SAMPLE_AUTH_CLIENT_ID'
         self.private_key_settings_name = 'SAMPLE_AUTH_PRIVATE_KEY'
-        self.client = Client()
+        self.client = TestableClient()
         self.domain = 'http://www.sample.com'
         self.path = '/some/path/'
         self.url = self.domain + self.path
@@ -73,13 +80,14 @@ class ClientTests(unittest.TestCase):
             getattr(self.client, client_property)
         self.assertIn(settings_name, error.exception.message)
 
-    @mock.patch('request_signer.client.generic.Request')
+    @mock.patch('generic_request_signer.request.Request')
     def test_get_response_creates_request(self, request):
         method = 'GET'
         data = dict(this="is", some='data', right='here')
+        request_kwargs = {"headers": {"Accept": "application/json"}}
         with mock.patch.object(Client, '_get_service_url') as get_url:
             get_url.return_value = 'my_url'
-            self.get_response(method, self.endpoint, data)
+            self.get_response(method, self.endpoint, data, **request_kwargs)
 
         request.assert_called_once_with(
             method,
@@ -87,26 +95,27 @@ class ClientTests(unittest.TestCase):
                 constants.CLIENT_ID_PARAM_NAME, self.client._client_id,
                 constants.SIGNATURE_PARAM_NAME, 'JjZFNxsw8HRZnuMlqGr_U4kLU_yHXxPpSm-3mbPdj9g='),
             None,
+            **request_kwargs
         )
 
-    @mock.patch('request_signer.client.generic.Request')
+    @mock.patch('generic_request_signer.request.Request')
     def test_get_response_creates_request_with_json_payload(self, request):
         method = 'POST'
         data = dict(this="is", some='data', right='here')
+        request_kwargs = dict(headers={"Content-Type": "application/json"})
         with mock.patch.object(Client, '_get_service_url') as get_url:
             get_url.return_value = 'my_url'
-            self.get_response(method, self.endpoint, data, content_type="application/json")
+            self.get_response(method, self.endpoint, data, **request_kwargs)
 
         request.assert_called_once_with(
             method,
             'my_url?{0}={1}&{2}={3}'.format(
                 constants.CLIENT_ID_PARAM_NAME, self.client._client_id,
-                constants.SIGNATURE_PARAM_NAME, '0R98AGFfWDnkwL4phL3ET3PJa-GVcy5ZC2mJPOEJFFc='),
+                constants.SIGNATURE_PARAM_NAME, 'KO_460pcuN_3mRfPBpLnTO9DbojZ2hf8bcr4-aptkN0='),
             json.dumps(data),
-            headers={'Content-Type': 'application/json'}
+            **request_kwargs
         )
 
-    @mock.patch('request_signer.client.generic.Request', mock.Mock())
     def test_get_response_gets_url_with_endpoint(self):
         with mock.patch.object(Client, '_get_service_url') as get_url:
             get_url.return_value = 'url'
@@ -153,7 +162,7 @@ class ClientTests(unittest.TestCase):
     def test_private_key_property_returns_private_key_from_settings(self):
         self.assertEqual(self.private_key, self.client._private_key)
 
-    @mock.patch('request_signer.client.generic.Request')
+    @mock.patch('generic_request_signer.request.Request')
     def test_get_raw_response_invokes_urlopen_with_request(self, request):
         self.get_response()
         self.urlopen.assert_called_once_with(request.return_value)
@@ -169,3 +178,26 @@ class ClientTests(unittest.TestCase):
         error_as_response = self.get_response()
         self.assertIsInstance(error_as_response, Response)
         self.assertEqual(expected, error_as_response.raw_response)
+
+
+class ClientBackendTests(test.TestCase):
+
+    def test_uses_api_credentials_from_authorized_service_provider_model(self):
+        prov = models.AuthorizedServiceProvider(base_url="my_url", client_id="my_id", private_key="my_key")
+        client = Client(api_credentials=prov)
+
+        self.assertEqual(prov.base_url, client._base_url)
+        self.assertEqual(prov.client_id, client._client_id)
+        self.assertEqual(prov.private_key, client._private_key)
+
+    def test_uses_api_credentials_from_object_that_implements_api_credential_attributes(self):
+        class X(object):
+            base_url = "my_url"
+            client_id = "my_id"
+            private_key = "my_key"
+
+        client = Client(api_credentials=X)
+
+        self.assertEqual(X.base_url, client._base_url)
+        self.assertEqual(X.client_id, client._client_id)
+        self.assertEqual(X.private_key, client._private_key)
