@@ -7,9 +7,6 @@ from django.core.management.base import CommandError
 from request_signer.management.commands import createclient
 from request_signer.models import create_private_key, AuthorizedClient
 
-__all__ = (
-    "CreateClientTests",
-)
 
 class CreateClientTests(test.TestCase):
 
@@ -29,7 +26,7 @@ class CreateClientTests(test.TestCase):
     def test_creates_client_with_key_provided(self):
         key = create_private_key()
 
-        self.command.handle(client="client", key=key)
+        self.command.execute(client="client", key=key)
 
         c = AuthorizedClient.objects.get(client_id="client")
         self.assertEqual(key, c.private_key)
@@ -38,9 +35,8 @@ class CreateClientTests(test.TestCase):
     def test_creates_new_private_key_when_not_given(self, create_private_key):
         create_private_key.return_value = "some private key"
 
-        self.command.handle(client="client")
+        self.command.execute(client="client")
 
         c = AuthorizedClient.objects.get(client_id="client")
         self.assertEqual("some private key", c.private_key)
         create_private_key.assert_called_once_with()
-
